#include <lights.h>
#include <pattern.h>
#include <joedog/joedog.h>
#include <sys/kd.h>
#include <setup.h>

/**
 * local prototypes
 */
int  word_count( char pattern, char *s );
int  verify_pattern( char *str, int len );
wackyd_led_change_t get_pattern( char *pattern );

/**
 * blink
 * rotates the lights from num to caps to scroll
 * with a 10000 microsecond delay in succession.
 */ 
int
blink( int console, SPEED microseconds )
{
  wackyd_led_change_t num;
  wackyd_led_change_t caps;
  wackyd_led_change_t scr;
  wackyd_led_change_t pattern[4];

  num.num = caps.cap = scr.scr = WACKYD_LED_TURN_ON;
  num.cap = num.scr = caps.num = caps.scr = scr.num = scr.cap = WACKYD_LED_TURN_OFF;
  num.sleep = caps.sleep = scr.sleep = microseconds;

  pattern[0] = num;
  pattern[1] = caps;
  pattern[2] = scr;
  pattern[3] = caps;
 
  for( ;my.done != TRUE; ){
    render_pattern(console, pattern, 4);
  }

  return 0;
}

int
custom_blink( int console, char **words, int len )
{
  int x;
  wackyd_led_change_t *pattern;
  pattern = (wackyd_led_change_t*)xmalloc( len * sizeof(wackyd_led_change_t));

  for( x = 0; x < len; x ++ ){
    pattern[x] = get_pattern( words[x] );
  }  
  for( ;my.done != TRUE; ){
    render_pattern(console, pattern, len );
  }
  for( x = 0; x < len; x++ ){
    free( words[x] );
  } 
  free( words );   
  pthread_exit( );
  return 0;
}

wackyd_led_change_t 
get_pattern( char *pattern )
{
  char *tmp;
  int  speed;
  wackyd_led_change_t led;

  /* assign optional speed value to temp */ 
  tmp      = substring( pattern, 3, strlen( pattern )); 
  speed    = atoi( tmp );
  led.num  = (TOLOWER(*(pattern+0))=='x')?WACKYD_LED_TURN_ON:WACKYD_LED_TURN_OFF;
  led.cap  = (TOLOWER(*(pattern+1))=='x')?WACKYD_LED_TURN_ON:WACKYD_LED_TURN_OFF;
  led.scr  = (TOLOWER(*(pattern+2))=='x')?WACKYD_LED_TURN_ON:WACKYD_LED_TURN_OFF;
  led.sleep= (speed==0)?300000:speed*100000;
  free( tmp ); 
  return led;
}

int
word_count( char pattern, char *s )
{
  int in_word_flag = 0;
  int count = 0;
  char *ptr;
 
  ptr = s;
  while( *ptr ){
    if(( *ptr ) != pattern ){
      if( in_word_flag == 0 )
        count++;
      in_word_flag = 1;
    }
    else
      in_word_flag = 0;
    ptr++;
  }
  return count;
} 

/**
 * esoteric function designed to verify
 * the integrity of the function, pattern
 * must match: [xo][xo][xo]NUM*
 */
int
verify_pattern( char *str, int len )
{
  char *walk, *save;
  char *tmp;
  int x;
 
  walk = save = str;
  save[len] = '\0';
  for( x=0; x<3; x++ ){
    if( ISX(*(walk+x)));
    else if( ISO(*(walk+x)));
    else{ return -1; }
  }
  tmp = substring( walk, x, len );
  if( atoi(tmp) < 0 ){
    joe_error( "Incorrect pattern, should be [xo][xo][xo]NUM,...\n" );
    free( tmp );
    return -1;
  }
  free( tmp );
  return 1;
}

char **
split( char pattern, char *s, int *n_words )
{
  char **words = NULL;
  char *str0, *str1;
  int i;
  *n_words = word_count( pattern, s);
 
  words = xmalloc( *n_words * sizeof (*words));
  if( !words )
    return NULL;
 
  str0 = s;
  i = 0;
  while( *str0 ){
    size_t len;
    str1 = strchr( str0, pattern );
    if( str1 != NULL ) 
      len = str1 - str0;
    else
      len = strlen( str0 ); 

    words[i] = xmalloc( len + 1 );
    if( !words[i] ){
      fprintf(
        stderr,
        "Warning: You got %d words out of %d.",
        i + 1, *n_words
      );
      return words;
    }
    if(( verify_pattern( str0, len ) < 0 )){
      printf( "error: bad pattern!" );
      free( words[i] );
      i--;
      *n_words = *n_words - 1;
    }
    else{
      strncpy( words[i], str0, len );
      words[i][len] = '\0';
    }
    if( str1 != NULL )
      str0 = ++str1;
    else
      break;
    i++;
  }
  return words;
} 
